var Game = function(data, uiHandler, soundHandler){
	this.ui = uiHandler;
	this.sound = soundHandler;
	this.gameData = data;
	this.questionsNumber = data.length;
	this.singleQuestionsNumber = 10;
	this.rounds = {
		roundsNumber: 3,
		currentRound: 1,
		finishedAllRounds : false
	}
	this.players = {
		number: 1
	};
	this.thisTurn = {
		count : 0,
		correctAnswer : ""
	};
	this.blocked = false;
	this.init();
}

Game.prototype.init = function(){
	var self = this;
	this.counter = new Counter({
		onCount: function(t){
			self.ui.updateTimerCounter(t);
		},
		onEndTime: function(){
			//self.guess("");
			self.sound.audioFiles.stop("timer");
			self.sound.audioFiles.play("bell");
		},
		reverse: true,
		timeLimit: 10	
	})
	
	$("#init").click(function(){		
 		self.sound.audioFiles.play("button");
		self.sound.audioFiles.stop("menu");
		self.sound.audioFiles.unload("menu");
		//self.sound.audioFiles.Menu.currentTime = 0; 
		
		self.setPlayers(self.players.number);
		
		self.ui.countDown(function(){
			self.start();
		});
		
		//self.ui.startMenu.css("display", "none");
		//self.ui.questionContainer.css("display", "block");
	});
	
	this.ui.calculateSize();	

	$(".start-turn-button").click(function(){
		self.sound.audioFiles.play("button")
		if(self.rounds.finishedAllRounds){
			self.endGame();
		}
		else {
			self.startTurn();
		}
	});
	
	$(".answer").click(function(){
		if(!self.blocked){
			self.guess($(this).find('span')[0].innerHTML);
			self.blocked = true;
		}
	});
	this.ui.aspectResize();
	//$(window).resize(this.ui.aspectResize);

	// Player Select Buttons
	$("#counter-left-button").mousedown(function(){
		self.sound.audioFiles.play("button")
		//cambia el numero de players
		if(self.players.number > 1){
			self.players.number--;
			self.ui.playersCounterNumber.attr('src', 'img/multiplayer_jugadores_' + self.players.number + ".png");
			$(this).attr('src', 'img/multiplayer_empezar_flecha_sumar.png');
		}
		
		if(self.players.number <= 1){ //el boton se pone gris cuando ya no debe ser tocado
			$(this).attr('src', 'img/multiplayer_empezar_flecha_sumar_lock.png');
			$(this).removeClass("button");
		}
		else if(self.players.number < 4){ //el boton se pone de color, para ser tocado
			$("#counter-right-button").attr('src', 'img/multiplayer_empezar_flecha_sumar.png');
			$("#counter-right-button").addClass("button");
		}
	});
	
	$("#counter-right-button").click(function(){
		self.sound.audioFiles.play("button")
		if(self.players.number < 4){
			self.players.number++;
			self.ui.playersCounterNumber.attr('src', 'img/multiplayer_jugadores_' + self.players.number + ".png");
			$(this).attr('src', 'img/multiplayer_empezar_flecha_sumar.png');
		}
		if(self.players.number >= 4){
			$(this).attr('src', 'img/multiplayer_empezar_flecha_sumar_lock.png');
			$(this).removeClass("button");
		}
		else if(self.players.number > 1){
			$("#counter-left-button").attr('src', 'img/multiplayer_empezar_flecha_sumar.png');
			$("#counter-left-button").addClass("button");
		}
	});
	
	// Exit prompt buttons
	$("#salir").click(function(){
 		self.sound.audioFiles.play("button");
 		self.sound.audioFiles.stop("timer");
		
		self.counter.pause();
		self.counter.pause();
		self.ui.showExitPrompt()
	});
	
	$("#exit-button-no").click(function(){
		self.ui.hideExitPrompt()
 		self.sound.audioFiles.play("button");
 		self.sound.audioFiles.loop("timer");
		
		self.counter.resume();
	});
	
	$("#exit-button-yes").click(function(){
		self.sound.audioFiles.play("button");
		self.counter.stop();
	
		location.reload();
	});
	
	// Starting Screen Buttons
	$("#start-game-button").click(function(){
		//self.sound.audioFiles.play("menu");
		self.sound.audioFiles.loop("menu");
		self.sound.audioFiles.play("button");
		$("#start-game-screen").hide();
		$("#player-select-menu").show();
		$("#logo").show();
		$("#salir").show();
	});
	
	// Podium Screen
	$("#podium-continue-button").click(function(){
		self.ui.hidePodium();
		$("#start-game-screen").show();
		location.reload();
	});
	
	// 1 Player Mode Scores Screen Button
	$("#total-score-button").click(function(){
		self.ui.hideTurnScore(self.players, false);
		self.ui.displayOnePlayerTotalScore(self.players.current.score);
	});
	
	// 1 Player Mode Total Score Screen Button
	$("#one-player-total-score-screen #continue-button").click(function() {
		location.reload();
	});
	
	// About Button
	$("#about-button").click(function(){
		$("#start-game-screen").hide();
		$("#about-screen").show();
	});
	
	// About Back-Button
	$("#about-screen #back-button").click(function(){
		$("#about-screen").hide();
		$("#start-game-screen").show();
	});

	this.ui.fixedPosition('start-game-button', true, true);
}

Game.prototype.start = function(){
	this.renderData();
	this.counter.start();
}

Game.prototype.guess = function(selectedAns){
  	this.sound.audioFiles.stop("timer");
	//this.sound.audioFiles.Timer.currentTime = 0;
	if(selectedAns == this.thisTurn.correctAnswer){
		this.guessCorrect();
	}else{
		this.guessIncorrect();
	}
	
	this.players.current.removeQuestion();

	this.thisTurn.count += 1;
	
	if(this.players.number > 1) {
		if(this.thisTurn.count > 2){ // if 3 turns have passed
			this.endTurn();
		}else if(this.rounds.currentRound <= this.rounds.roundsNumber){ // if we are still in any of the available rounds
			this.onAnswer(true);
		}
	}
	else if(this.thisTurn.count >= this.singleQuestionsNumber) {
		this.endTurn();
	}
	else {
		this.onAnswer(true);
	}
}

Game.prototype.renderData = function(){
	var self = this;
	this.ui.drawQuestion(this.getQuestion());
	this.ui.drawAnswers(this.getAnswers(), function(){
		self.blocked = false;
	});
	this.ui.displayTimerCounter();
	this.sound.audioFiles.loop("timer");
	this.ui.drawPlayer(this.players.current.number);
}

Game.prototype.guessCorrect = function(){
	var time = this.counter.pause();
  	if(time > 7){
		this.sound.audioFiles.play("guessPerfect");
	}else{
		this.sound.audioFiles.play("guessRight");
	} 
	this.addScore(time);
	this.ui.displayTrafficLights(true);
}

Game.prototype.guessIncorrect = function(){
	this.counter.pause();
	this.addScore(0);
	this.sound.audioFiles.play("guessWrong");
	this.ui.displayTrafficLights(false);
}

Game.prototype.getQuestion = function(){
	return this.players.current.gameDataCopy[0].pregunta;
}

Game.prototype.endGame = function(){
	// show podium
	this.ui.displayPodium(this.players, this.getPlayersScore(true));
}

Game.prototype.getAnswers = function(){
	this.thisTurn.correctAnswer = this.players.current.gameDataCopy[0].respuestas[1];
	var toArray = $.map(this.players.current.gameDataCopy[0].respuestas, function(value, index) {
	 	return [value];
	});
	this.ui.drawAnswers(this.shuffle(toArray), function(){
		self.blocked = false;
	});
}

Game.prototype.setPlayers = function(n){
	//this.players.number = n;
	for(var i = 0; i < n; i++){
		this.players[i + 1] = new Player(i + 1, this.shuffle($.extend(true, [], this.gameData)));
	}
	this.players.current = this.players[1];
}

Game.prototype.nextPlayer = function(){
	// restart the question counter
	this.players.current.clearQuestionScores();

	var nextPlayerNumber = this.players.current.number + 1;
	
	if(parseInt(nextPlayerNumber) <= parseInt(this.players.number) ){
		this.players.current = this.players[nextPlayerNumber];
	}else if(parseInt(nextPlayerNumber) > parseInt(this.players.number) ){
		this.players.current = this.players[1];
		this.rounds.currentRound += 1;
	}
}

Game.prototype.endTurn = function(){
	// we wait a little so that the user can see the result of his last question guess
	window.setTimeout(function(game){		
		game.ui.displayTurnScore(game.players, game.rounds, game.getPlayersScore(true));
		
		// check if this was the final player of the final round
		if(parseInt( game.players.current.number + 1 ) > parseInt( game.players.number ) && game.rounds.currentRound+1 > game.rounds.roundsNumber){
			game.rounds.finishedAllRounds = true;
		}
		else {
			game.nextPlayer();
			game.thisTurn.count = 0;
			game.onAnswer();		
		}
		
	}, 1600, this);
}

Game.prototype.startTurn = function(){
	this.ui.hideTurnScore(this.players, true);
	this.nextQuestion();
}

Game.prototype.onAnswer = function(next){
	game.counter.stop();
	
	window.setTimeout(function(game){
		if(next){
			game.nextQuestion();
		}
	}, 2000, this);
	window.setTimeout(function(game){
		game.ui.hideTrafficLights();
		game.ui.hideAnswers()
	}, 1600, this);
}
Game.prototype.nextQuestion = function(){
	this.renderData();
	this.counter.start();
}

Game.prototype.addScore = function(t){
	this.players.current.score += t * 100;
	var n = this.players.current.questionNumber++;
	this.players.current.perQuestionScore[n] = t * 100;
}

Game.prototype.shuffle = function(o){ 
	for(var j, x, i = o.length; i; j = parseInt(Math.random() * i), x = o[--i], o[i] = o[j], o[j] = x);
	return o;
};

Game.prototype.getPlayers = function(){
	var result = [];
	for(p in this.players){
		if(!isNaN(parseInt(p))){
			result.push(this.players[p]);
		}
	}
	return result;
}

//If parameter TRUE it will order the result by score, if FALSE it will
//do it by player
Game.prototype.getPlayersScore = function(orderByScore){
	var players = this.getPlayers();
	var scores = [];
	var asocScores = {};
	var arrayOfAsoc = [];

	for(p in players){
		scores.push(players[p].score);  
	}
	
	function sortNumber(a,b) {
    	return b-a;
	}

	scores.sort(sortNumber);
	
	for(s in scores){
		for(pl in players){
			if(scores[s] == players[pl].score)
				arrayOfAsoc.push({
					player: players[pl].number,
					score: players[pl].score
				})
				asocScores[players[pl].number] = players[pl].score;
		}
	}
	if(orderByScore){
		return arrayOfAsoc;
	}else{
		return asocScores;
	}
}



